package com.tzld.piaoquan.recommend.server.service.filter.strategy;

import com.tzld.piaoquan.recommend.server.service.filter.FilterParam;
import com.tzld.piaoquan.recommend.server.service.filter.FilterStrategy;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Component
public class RiskVideoStrategy implements FilterStrategy {

    @Override
    public List<Long> filter(FilterParam param) {

        if (CollectionUtils.isEmpty(param.getVideoIds())) {
            return Collections.emptyList();
        }
        boolean riskFlag = param.getRiskFilterFlag();
        int appType = param.getAppType();
        String regionCode = param.getRegionCode();
        Map<Integer, List<String>> rules = param.getAppRegionFiltered();
        List<Long> videosWithRisk = param.getVideosWithRisk();
        List<Long> videoIds = param.getVideoIds();


        if (!riskFlag) {
            return this.truncation(param.getVideoIds(), param.getForceTruncation());
        }
        // 1 判断是否过滤,不展示的app+区域列表。
        boolean filterFlag = false;
        if (rules.containsKey(appType)) {
            filterFlag = false;
            if (rules.get(appType).contains(regionCode)) {
                filterFlag = true;
            }
        } else {
            filterFlag = true;
        }
        if (!filterFlag) {
            return this.truncation(videoIds, param.getForceTruncation());
        }
        // 2 开始过滤。
        List<Long> videoIdNew = new ArrayList<>();
        for (Long videoId : videoIds) {
            if (!videosWithRisk.contains(videoId)) {
                videoIdNew.add(videoId);
            }
        }
        return this.truncation(videoIdNew, param.getForceTruncation());
    }

    private List<Long> truncation(List<Long> videoIds, int forceTruncation) {
        if (forceTruncation == 0) {
            return videoIds;
        } else {
            return videoIds.subList(0, Math.min(forceTruncation, videoIds.size()));
        }
    }
}
